define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/presentation/ui/imagecrop-test',
    'testsuite/sinon',
    'jquery',
    'de_epages/presentation/ui/imagecrop',
    'ep',

    '$ready!'
], function (QUnit, fixture, sinon, $, de_epages, ep) {
    // add css for cropper framework
    ep('head').append('<link rel="stylesheet" type="text/css" href="../jquery/cropperjs/cropper.css">');

    var cropBoxSelector = '.cropper-crop-box',
        aspectRatioMap = {
            '1:1': {method: 'setAspectRatio', label: '1:1'},
            '2:3': {method: 'setAspectRatio', label: '2:3'},
            '4:3': {method: 'setAspectRatio', label: '4:3'},
            '16:9': {method: 'setAspectRatio', label: '16:9'},
            '3:2': {method: 'setAspectRatio', label: '3:2'}
        },

        rotateMap = {
            'acw-90': {method: 'rotate', label: '-90°'},
            'cw-90': {method: 'rotate', label: '90°'}
        },

        navigationButtonMap = {
            'zoom-plus': {method: 'zoom'},
            'zoom-minus': {method: 'zoom'},
            'move-left': {method: 'move'},
            'move-right': {method: 'move'},
            'move-up': {method: 'move'},
            'move-down': {method: 'move'},
            'reset': {method: 'reset'}
        };

    QUnit.module('de_epages.presentation.ui.imagecrop', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            '_resizeImage',
            '_createDialog',
            '_resetUI',
            '_destroyData',
            '_getCropData',
            '_initImageCrop',
            '_handleCrop',
            '_setCropDimensionInputs',
            '_handleDisableCrop',
            '_handleAspectRatioButton',
            '_enableAspectRatioCustom',
            '_disableAspectRatioCustom',
            '_handleAspectRatioLock',
            '_handleCropWidth',
            '_handleCropHeight',
            '_handleButton',
            '_handleCropData',
            'open',
            'close',
            'destroy'
        ], function (i, fn) {
                QUnit.equal(
                $.isFunction($.ui.presentationUiImageCrop.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("widget ui tests", function () {

        /* START - fake ajax request */
        ep.ajax = function (o) {
            var result = $.Deferred(),
                cropData;

            if (o.data && o.data.ChangeAction) {
                if (o.data.ChangeAction === "JSONSaveCropData") {
                    cropData = JSON.parse(o.data.cropData);

                    // test if sent data are correct
                    QUnit.ok(
                        cropData.width === newWidth,
                        'ep.ajax: Correct crop width was sent.'
                    );
                    QUnit.ok(
                        cropData.height === newHeight,
                        'ep.ajax: Correct crop height was sent.'
                    );
                    QUnit.ok(
                        cropData.rotate === 90,
                        'ep.ajax: Correct rotation angle was sent.'
                    );
                    QUnit.ok(
                        cropData.new === true,
                        'ep.ajax: Cropped a newly uploaded image.'
                    );
                    QUnit.ok(
                        cropData.name.indexOf("?") === -1,
                        'ep.ajax: Timestamp was removed from image name.'
                    );

                    result.resolve({
                        success: true
                    });
                }
            } else {
                result.reject();
            }

            return result;
        };
        /* END - fake ajax request */

        var $cropButton = $('.ep-dropzone-crop-image'),
            $image = $('.ep-dropzone-sortable-item-holder img'),
            id = $image.data('id'),
            newProduct = false,
            resizeSpy,
            destroySpy,
            inputCustomWidth,
            inputCustomHeight,
            dataUrl,
            basicCanvasData,
            canvasData,
            newWidth = 200,
            newHeight = 190;

        // register spies on widget methods => number of executions can be investigated
        resizeSpy = sinon.spy($.ui.presentationUiImageCrop.prototype, '_resizeImage');
        destroySpy = sinon.spy($.ui.presentationUiImageCrop.prototype, '_destroyData');
        // set input delay to 10 milliseconds for test reasons
        $.ui.presentationUiImageCrop.prototype.inputDelay = 10;


        $cropButton.presentationUiImageCrop({
            img: $image,
            id: id,
            objectID: 1,
            resize: !newProduct,
            resizeDim: 250,
            // dialog: {
            //     cta: 'Apply'
            // },
            onAfterCrop: function(opts) {
                // save dataUrl
                dataUrl = opts.dataUrl;

                // new products: find input for dataUrl
                var input = $('#ep-dropzone-inputs-' + id).find('input[name=dataUrl]');

                // if an input for dataUrl exists and a dataUrl was handed over
                if (input.length && opts.dataUrl) {
                    // set changed dataUrl
                    input.val(opts.dataUrl);

                    // if a new
                    if (newProduct) {
                        QUnit.ok(
                            true,
                            'Found correct hidden input to put in new dataUrl (relevant for new products)'
                        );
                    }
                }
            },
            onInterruptSave: (function() {
                return newProduct
                    ? function(data) {
                            // example data structure:
                            // {"cropData":{"x":100,"y":200,"width":800,"height":600,"rotate":180,"scaleX":1,"scaleY":1,"id":"id_51","name":"image.jpg","new":true}}
                        }
                    : undefined;
            })()
        });

        setTimeout(function() {
            QUnit.ok(
                $cropButton.data('uiPresentationUiImageCrop') !== undefined,
                'Widget exists'
            );

            // Trigger click-event to open dialog
            $cropButton.trigger('click');

            setTimeout(function() {
                QUnit.ok(
                    resizeSpy.callCount === 1,
                    'Resize method for images was executed during widget creation.'
                );

                QUnit.ok(
                    Math.max(
                        $cropButton.data('uiPresentationUiImageCrop').tmplElements.image[0].naturalWidth,
                        $cropButton.data('uiPresentationUiImageCrop').tmplElements.image[0].naturalHeight
                    ) === 250,
                    'Test image resize: image was resized to a maximum value of 250px'
                );
                QUnit.ok(
                    $cropButton.data('uiPresentationUiImageCrop').dialog !== undefined,
                    'Crop dialog instance exists'
                );
                QUnit.ok(
                    $cropButton.data('uiPresentationUiImageCrop').dialog.is(':visible') ,
                    'Crop dialog is visible'
                );

                // test presence of navigation and tool buttons
                QUnit.ok(
                    $('.ep-imagecrop-content-navigationbar-button').length === Object.keys(navigationButtonMap).length,
                    Object.keys(navigationButtonMap).length + ' navigation buttons are present.'
                );
                QUnit.ok(
                    $('.ep-imagecrop-toolbar-button[data-method="clear"]').length === 1,
                    'Button to disable cropping is present.'
                );
                QUnit.ok(
                    $('.ep-imagecrop-toolbar-button[data-method="rotate"]').length === 2,
                    'Two buttons for rotation are present.'
                );
                Object.keys(aspectRatioMap).forEach(function(key) {
                    QUnit.ok(
                        $('.ep-imagecrop-toolbar-button[data-aspect-ratio-current="'+key+'"]') !== undefined,
                        'Aspect ratio button "' + key + '" is present.'
                    );
                });

                // test if crop box is visible on start up
                QUnit.ok(
                    $(cropBoxSelector).is(':visible'),
                    'Crop box is visible.'
                );

                // check base function of navigation buttons
                basicCanvasData = $cropButton.data('uiPresentationUiImageCrop').tmplElements.image.cropper('getCanvasData');

                // move canvas 10px left and 10px down
                for (var i=0; i<5; i++) {
                    $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(2).trigger('click');
                    $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(5).trigger('click');
                }

                // move canvas 5px right and 5px up
                for (var i=0; i<10; i++) {
                    $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(3).trigger('click');
                    $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(4).trigger('click');
                }

                canvasData = $cropButton.data('uiPresentationUiImageCrop').tmplElements.image.cropper('getCanvasData');

                QUnit.equal(
                    canvasData.left - basicCanvasData.left,
                    5,
                    "Test navigation buttons (1. move 5px left, 2. move 10px right): move canvas 5 pixels to right side"
                );
                QUnit.equal(
                    canvasData.top - basicCanvasData.top,
                    -5,
                    "Test navigation buttons (1. move 5px down, 2. move 10px up): move canvas 5 pixels up"
                );

                // zoom in
                $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(0).trigger('click');
                QUnit.equal(
                    basicCanvasData.width * 1.1,
                    $cropButton.data('uiPresentationUiImageCrop').tmplElements.image.cropper('getCanvasData').width,
                    "Test navigation buttons (zoom in): cavas was zoomed in by the factor 0.1"
                );
                // zoom out
                $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(1).trigger('click');
                QUnit.equal(
                    basicCanvasData.width,
                    $cropButton.data('uiPresentationUiImageCrop').tmplElements.image.cropper('getCanvasData').width,
                    "Test navigation buttons (zoom out): cavas was zoomed out by the factor 0.1"
                );

                // reset canvas
                $('.ep-imagecrop-content-navigationbar .ep-imagecrop-content-navigationbar-button').eq(6).trigger('click');
                QUnit.ok(
                    JSON.stringify($cropButton.data('uiPresentationUiImageCrop').tmplElements.image.cropper('getCanvasData')) === JSON.stringify(basicCanvasData),
                    "Test navigation buttons (reset): canvas was resetted correctly"
                );

                setTimeout(function() {
                    $('.ep-imagecrop-toolbar-button[data-method="clear"]').trigger('click');

                    // test if crop box is hidden after click the disable button
                    QUnit.ok(
                        !$(cropBoxSelector).is(':visible'),
                        'Crop box is hidden.'
                    );

                    // rotate 90 degree anti clockwise first and 180 degree clockwise next => 90 degree
                    $('.ep-imagecrop-toolbar-button[data-method="rotate"]').eq(0).trigger('click');
                    $('.ep-imagecrop-toolbar-button[data-method="rotate"]').eq(1).trigger('click');
                    $('.ep-imagecrop-toolbar-button[data-method="rotate"]').eq(1).trigger('click');

                    inputCustomWidth = $('.ep-imagecrop-toolbar-custom-width');
                    inputCustomHeight = $('.ep-imagecrop-toolbar-custom-height');
                    QUnit.ok(
                        inputCustomWidth.prop('disabled'),
                        'Input field for width is disabled.'
                    );
                    QUnit.ok(
                        inputCustomHeight.prop('disabled'),
                        'Input field for height is disabled.'
                    );

                    // enable custom cropping
                    $('.ep-imagecrop-toolbar-button-custom').trigger('click');
                    $('.ep-imagecrop-toolbar-aspectratio-lock').trigger('click');

                    QUnit.ok(
                        !inputCustomWidth.prop('disabled'),
                        'Input field for width is enabled.'
                    );
                    QUnit.ok(
                        !inputCustomHeight.prop('disabled'),
                        'Input field for height is enabled.'
                    );

                    inputCustomWidth.val(newWidth);
                    inputCustomWidth.trigger('input');

                    // wait for input delay (set to 10 ms for tests, 300 ms in production)
                    setTimeout(function() {
                        inputCustomHeight.val(newHeight);
                        inputCustomHeight.trigger('input');

                        // wait for input delay (set to 10 ms for tests, 300 ms in production)
                        setTimeout(function() {
                            // close dialog
                            $('.ep-imagecrop-dialog .epDialogButtonBar .ep-button-cta').trigger('click');

                            setTimeout(function() {
                                QUnit.ok(
                                    destroySpy.callCount === 1,
                                    'Dialog UI was destroyed on close event.'
                                );

                                QUnit.ok(
                                    $('.ep-dropzone-sortable-item-holder img').attr('src') === dataUrl,
                                    'Source (src) of image is the same as the result of the cropping.'
                                );
                                QUnit.ok(
                                    $('#ep-dropzone-inputs-' + id).find('input[name=dataUrl]').val() === dataUrl,
                                    'onAfterCrop: Correct value (dataUrl) was written into the related hidden input.'
                                );

                                // simulate new product
                                newProduct = true;
                                // check onInterruptSave callback
                                $cropButton.presentationUiImageCrop('option', 'onInterruptSave', function(data) {
                                    // test if sent data are correct
                                    QUnit.ok(
                                        Math.round(data.cropData.width) === 160,
                                        'onInterruptSave: Correct crop width are present.'
                                    );
                                    QUnit.ok(
                                        Math.round(data.cropData.height) === 107,
                                        'onInterruptSave: Correct crop height are present.'
                                    );
                                    QUnit.ok(
                                        Math.round(data.cropData.naturalWidth) === newWidth,
                                        'onInterruptSave: Orignal width of the image was ' + newWidth + 'px.'
                                    );
                                    QUnit.ok(
                                        Math.round(data.cropData.naturalHeight) === newHeight,
                                        'onInterruptSave: Orignal height of the image was ' + newHeight + 'px.'
                                    );
                                    QUnit.ok(
                                        data.cropData.rotate === 0,
                                        'onInterruptSave: Correct rotation angle are present.'
                                    );
                                    QUnit.ok(
                                        data.cropData.new === true,
                                        'onInterruptSave: Cropped an already uploaded image.'
                                    );
                                });

                                // set new data flag to true
                                $('.ep-dropzone-sortable-item-holder img').data("new", true);

                                // Trigger click-event to open dialog
                                $cropButton.trigger('click');

                                setTimeout(function() {
                                    // cropping tests for new product are done in the onInterruptSave tests

                                    // close dialog
                                    $('.ep-imagecrop-dialog .epDialogButtonBar .ep-button-cta').trigger('click');

                                    setTimeout(function() {
                                        QUnit.ok(
                                            destroySpy.callCount === 2,
                                            'Dialog UI was destroyed again on close event.'
                                        );

                                        // destroy image crop widget
                                        $cropButton.presentationUiImageCrop('destroy');

                                        setTimeout(function() {
                                            QUnit.ok(
                                                $cropButton.data('uiPresentationUiImageCrop') === undefined,
                                                'Widget destroyed.'
                                            );

                                            // start async tests
                                            QUnit.start();
                                        }, 10);
                                    }, 10);
                                }, 100);
                            }, 10);
                        }, 20);
                    }, 20);
                }, 100);
            }, 400);
        }, 500);
    });
});
