/*globals require, window*/

require([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/shop/ui/dashboard-test',

    'de_epages/shop/ui/dashboard',
    'de_epages/shop/ui/dashboardwidget'
], function ($, QUnit, fixture) {
    'use strict';

    $.ajax = function (o) {
        var result = $.Deferred();
        if (o.data && o.data.ChangeAction === 'JSONAddWidget') {
            result.resolveWith(o.context || result, [{
                "widgetId": "17"
            }]);
        } else if (o.data && o.data.ChangeAction === 'JSONSaveWidgetPositions') {
            result.resolveWith(o.context || result, [{}]);
        } else if (o.data && o.data.ChangeAction === 'JSONRemoveWidget') {
            result.resolveWith(o.context || result, [{}]);
        } else if (o.data && o.data.ViewAction === 'DummyReloadAction') {
            result.resolveWith(o.context || result, ['ReloadedContent']);
        } else if (o.data && o.data.ViewAction === 'CrashDummyReloadAction') {
            result.rejectWith(o.context || result);
        } else if (o.data && o.data.ViewAction === 'JSONSnippet' && o.data.TemplateAction && o.data.TemplateAction === 'MBO-ViewAddWidgetDialogSnippet') {
            result.resolveWith(o.context || result, [{
                'snippet': '<div id="DashboardDialog"><div class="ui-tabs-left ep-js" data-js="ui-tabs ui-widget ui-widget-content ui-corner-all"><ul class="ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all"><li class="WidgetGroup_1784 ui-state-default ui-corner-top ui-tabs-selected ui-state-active"><a title="" href="#WidgetGroup_1784">Funktionen</a></li><li class="WidgetGroup_1785 ui-state-default ui-corner-top"><a title="" href="#WidgetGroup_1785">Besucheraktionen</a></li></ul><div id="WidgetGroup_1784" class="ui-tabs-panel ui-widget-content ui-corner-bottom"><ul></ul></div><div id="WidgetGroup_1785" class="ui-tabs-panel ui-widget-content ui-corner-bottom ui-tabs-hide"><ul></ul></div></div></div>'
            }]);
        }
        return result;
    };

    QUnit.module('de_epages/shop/ui/dashboard', {
        setup: function () {
            fixture.init();

            var objectIds = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16],
                widgetCloneNode = $('.ep-DashboardWidget').clone(),
                reloadNode,
                reloadNode2,
                i;

            for (i = 0; i < 3; i += 1) {
                widgetCloneNode.clone().appendTo('.Dashboard');
            }

            $('.ep-DashboardWidget').each(function () {
                $(this).shopUiDashboardWidget({
                    'dashboard': '.Dashboard',
                    'objectId': objectIds.shift(),
                    'widgetTypeId': 500
                });
            });

            reloadNode = widgetCloneNode.clone().appendTo('.Dashboard');
            $(reloadNode).shopUiDashboardWidget({
                'dashboard': '.Dashboard',
                'objectId': objectIds.shift(),
                'widgetTypeId': 500,
                'reload': true,
                'action': 'DummyReloadAction'
            });

            reloadNode2 = widgetCloneNode.clone().appendTo('.Dashboard');
            $(reloadNode2).shopUiDashboardWidget({
                'dashboard': '.Dashboard',
                'objectId': objectIds.shift(),
                'widgetTypeId': 510,
                'reload': true,
                'action': 'CrashDummyReloadAction'
            });

            $('.Dashboard').shopUiDashboard({
                userId: '815'
            });
        },

        teardown: function () {
            fixture.destroy();
        }
    });

    QUnit.asyncTest('de_epages/shop/ui/dashboard', function () {

        window.setTimeout(function () {
            QUnit.equal($('.Dashboard.ui-sortable').length, 1, 'Dashboard created');
            QUnit.equal($('.ep-DashboardWidget').length, 6, 'Dashboard Widget count correct');

            $('.ep-DeleteButtonNode:first').trigger('mouseover');
            $('.ep-DeleteButtonNode:first').trigger('mouseout');
            $('.ep-DeleteButtonNode:first').trigger('mousedown');
            $('.ep-DeleteButtonNode:first').trigger('click');

            window.setTimeout(function () {
                QUnit.equal($('.ep-DashboardWidget').length, 5, 'Dashboard Widget was deleted correct');

                $('.ep-AddButtonNode:eq(1)').trigger('click');
                QUnit.equal($('.ep-DashboardWidget').length, 6, 'Dashboard Widget was added correct');

                $('.Dashboard').trigger('showAddWidgetDialog');
                QUnit.equal($('.ui-dialog').length, 1, 'Dialog opened');

                $('.ui-dialog-titlebar-close').trigger('click');
                $('.ep-DashboardWidget:first').data('uiShopUiDashboardWidget').options.objectId = undefined;
                $('.ep-DeleteButtonNode:first').trigger('click');
                QUnit.equal($('.ep-DashboardWidget').length, 5, 'Dashboard Widget without objectId was deleted');

                $('.ep-DashboardWidget').shopUiDashboardWidget('destroy');
                QUnit.equal($('.ep-DashboardWidget').length, 0, 'Dashboard Widgets destroyed');

                QUnit.start();

            }, 600);

            $('.ep-DashboardWidget:eq(2)').shopUiDashboardWidget('setStatus', 'preview');
            QUnit.equal($('.ep-DashboardWidget:eq(2) .ep-PreviewLayerAddNode:visible').length, 1, 'status preview checked');

            $('.ep-DashboardWidget:eq(1)').shopUiDashboardWidget('setStatus', 'used');
            QUnit.equal($('.ep-DashboardWidget:eq(1) .ep-PreviewLayerNode:visible').length, 1, 'status used checked');

        }, 500);
    });
});